﻿using System;
using System.Collections;

using CMS;
using CMS.Base;
using CMS.EcommerceProvider;
using CMS.Helpers;
using Braintree;
using CMS.Membership;
using CMS.Globalization;
using CMS.DataEngine;

[assembly: RegisterCustomClass("BraintreePaymentGateway", typeof(Custom.BraintreePaymentGateway))]

namespace Custom
{
    public class BraintreePaymentGateway : CMSPaymentGatewayProvider
    {
        public static BraintreeGateway gateway = new BraintreeGateway
        {
            Environment = SettingsKeyInfoProvider.GetBoolValue("BraintreeTestMode") ? Braintree.Environment.SANDBOX : Braintree.Environment.PRODUCTION,
            PublicKey = ValidationHelper.GetString(SettingsKeyInfoProvider.GetSettingsKeyInfo("BraintreePublicKey").KeyValue, ""),
            PrivateKey = ValidationHelper.GetString(SettingsKeyInfoProvider.GetSettingsKeyInfo("BraintreePrivateKey").KeyValue, ""),
            MerchantId = ValidationHelper.GetString(SettingsKeyInfoProvider.GetSettingsKeyInfo("BrainTreeMerchantID").KeyValue, "")
        };

        /// <summary>
        /// Returns path to payment gateway form with custom controls.
        /// </summary>
        public override string GetPaymentDataFormPath()
        {
            return "~/DancingGoat/Controls/BraintreePaymentForm.ascx";
        }

        /// <summary>
        /// Process payment.
        /// </summary>
        public override void ProcessPayment()
        {
            try
            {
                var country = CountryInfoProvider.GetCountryInfo(Order.OrderBillingAddress.AddressCountryID);

                // Create the transaction
                TransactionRequest transactionRequest = new TransactionRequest
                {
                    Amount = ValidationHelper.GetDecimal(Order.OrderTotalPrice, 0),
                    CreditCard = new TransactionCreditCardRequest
                    {
                        CardholderName = ValidationHelper.GetString(PaymentDataForm.PaymentGatewayCustomData["CardName"], ""),
                        Number = ValidationHelper.GetString(PaymentDataForm.PaymentGatewayCustomData["CardNumber"], ""),
                        CVV = ValidationHelper.GetString(PaymentDataForm.PaymentGatewayCustomData["CVV2"], ""),
                        ExpirationMonth = ValidationHelper.GetString(PaymentDataForm.PaymentGatewayCustomData["ExpMonth"], ""),
                        ExpirationYear = ValidationHelper.GetString(PaymentDataForm.PaymentGatewayCustomData["ExpYear"], ""),
                    },
                    BillingAddress = new AddressRequest
                    {
                        StreetAddress = Order.OrderBillingAddress.AddressLine1,
                        ExtendedAddress = Order.OrderBillingAddress.AddressLine2,
                        Locality = Order.OrderBillingAddress.AddressCity,
                        CountryName = country.CountryTwoLetterCode,
                        PostalCode = Order.OrderBillingAddress.AddressZip
                    }
                };

                // Post to Braintree and get the result
                Result<Transaction> result = gateway.Transaction.Sale(transactionRequest);

                //Check if the transaction was successful
                if (result.IsSuccess())
                {
                    PaymentResult.PaymentIsCompleted = true;
                    PaymentResult.PaymentTransactionID = result.Target.Id;
                    PaymentResult.PaymentStatusName = "{$PaymentGateway.Result.Status.Completed$}";
                    PaymentResult.PaymentStatusValue = "completed";
                    ErrorMessage = "";
                    InfoMessage = "Payment succeeeded!";
                }
                else
                {
                    ErrorMessage += result.Message;
                }
            }
            catch (Exception ex)
            {
                ErrorMessage = "Unable to finish payment: " + ex.Message;
            }

            if (ErrorMessage != "")
            {
                // Update payment result
                PaymentResult.PaymentDescription = ErrorMessage;
                PaymentResult.PaymentStatusName = "{$PaymentGateway.Result.Status.Failed}";
                PaymentResult.PaymentStatusValue = "failed";
                PaymentResult.PaymentIsCompleted = false;
            }
            
            // Update order payment result in database
            UpdateOrderPaymentResult();
        }
    }
}